#ifndef _XML_TEMPLATE_H
#define _XML_TEMPLATE_H 1

#include <libxml/tree.h>

#include <string>
#include <utility>
#include <unordered_map>
#include <vector>

class Directive {
 public:
	virtual ~Directive();
	virtual void process(xmlNode *node, bool clean) = 0;
	virtual std::string get_contents();  // Only makes sense for Replace.
};

class Replace : public Directive {
 public:
	Replace(const std::string &str);
	virtual void process(xmlNode *node, bool clean);
	virtual std::string get_contents();

 private:
	const std::string str;
};

class Clone : public Directive {
 public:
	Clone(const std::vector<Directive *> &subdirectives);
	Clone(std::initializer_list<Directive *> subdirectives);
	~Clone();
	virtual void process(xmlNode *node, bool clean);

 private:
	const std::vector<Directive *> subdirectives;
};

class Substitute : public Directive {
 public:
	Substitute(const std::unordered_map<std::string, Directive*> &substitution_map);
	Substitute(std::initializer_list<std::pair<const std::string, Directive*>> substitution_map);
	~Substitute();

	virtual void process(xmlNode *node, bool clean);

 private:
	const std::unordered_map<std::string, Directive*> substitution_map;
};

void process_file(const std::string &input_filename,
                  const std::string &output_filename,
                  Directive *root_directive);

#endif  // !defined(_XML_TEMPLATE_H)
